#ifndef _Global_H_
#define _Global_H_

#include <vector>
#include <deque>
#include <Magick++.h>
#include "JPath.h"
#include "JImage.h"
#include "Drip.h"
#include "ColorFingerPrint.h"
#include "FingerPrint.h"
#include "FingerPoints.h"

using namespace std;
using namespace Magick;

using namespace grl;

/**
   This is the main, entry level class that gets instantiated. I
   generally populate this class when I am sketching anew. I also tend
   to clean this class out a lot when moving to a new project.
*/
class Global{
 public:
  
  /**
     the one and only static global instance of the Global object.
     refer to me as `Global::instance`
  */
  deque<JoshFont*>fonts;
  
  /**
     a list of graffitti screen drips that exist somewhat indifferenty
     to the drawn vector geometries. They just sample the screen and
     do their jobs.
  */
  deque<Drip*> drips;

  /**
     a list of paths drawn by each mouse drag event.
  */
  deque<JPath*> paths;
  
  

  static Global*instance; ///< sort of like _global or _root in flash so you can say Global::instance->drips from anywhere.
  void loop(); ///< the loop callback called by the system. Equiv to Processing draw()
  void keyDown(int k,int special); ///< catch a keyDown event by populating this function with creative code!
  void keyUp(int k,int special); ///< catch a keyUp event by populating this function with creative code!
  
  void mouseDown(); ///< catch a mouseDown event by populating this function with creative code!
  void mouseUp();///< catch a mouseUp event.
  Global();
  ~Global();
  bool mouseIsDown; ///< i want to keep track of whether or not the mouse is down.
  int breakWait; ///< how long to wait before breaking away - i'm not sure if this is quite working yet.
  int lastMouseUpTime; ///< time since the last mouseup event.
  
  bool rotating;  ///< are we using the mouse to rotate?
  float lastMouseX, lastMouseY;
  
  ColorFingerPrint rgb; ///< my collection of self-providing persistent random colors.
  int drips_max; ///< how many drips on the screen at most?
  void resetAndShuffle(); ///< change the brush.
  void zapFirstPath(); ///< remove first path
  FingerPrint rnd; ///< my collection of self-providing persistent random floated values between 0 and 1.
  FingerPoints rndpoints; ///< my collection of self-providing persistent random jpoints.
  deque<JImage*> blots; ///< a flat pool of little spray blots.
  deque<JImage*> stars; ///< a flat pool of little spray stars.
  void killAllDrops(); ///< deallocate all drips.
  bool grl_logo_easter_egg;///< whether or not to render the spinning GRL logo for the multimedia rebuttle.
};

#endif

