#include "JImage.h"
#include "gl_et_al.h"
#include "jttoolkit.h"


void JImage::initMem(){
  listID=0;
}


void JImage::loadMemory(unsigned char *pix,int w,int h){
  //read("media/backplate.png");
  read(w,h,"RGB",CharPixel,pix);
  modifyImage();
  type(TrueColorType);
  pixels = getPixels(0,0,columns(),rows()); 
  update();
}



void JImage::load(char *fname){
  read(fname);
  modifyImage();
  type(TrueColorType);
  pixels = getPixels(0,0,columns(),rows()); 
  update();
}

JImage::JImage(){
  initMem();
}

JImage::JImage(char *file){
  initMem();
  load(file);
}

void JImage::update(){
  update(false);
}



int JImage::getNextHighestGLTexSize(){
  int v = max(rows(),columns());
  if(v>1 && v<=2)return 2;
  else if(v>2 && v<=4)return 4;
  else if(v>4 && v<=8)return 8;
  else if(v>8 && v<=16)return 16;
  else if(v>16 && v<=32)return 32;
  else if(v>32 && v<=64)return 64;
  else if(v>64 && v<=128)return 128;
  else if(v>128 && v<=256)return 256;
  else if(v>256 && v<=512)return 512;
  else if(v>512 && v<=1024)return 1024;
  else throw 0;
}

/**
   when the gl texture is generated, it needs to be square mult of 2.
*/
void JImage::update(int alphaOnly){
  syncPixels();
  //kill it first if needed
  deleteGLResources();
  
  int gl_columns = getNextHighestGLTexSize();
  int gl_rows = gl_columns;
  

  //buffer for re-ordering pixels for opengl color space.
  GLfloat *f = new GLfloat[gl_columns*gl_rows*4];



  // first fill the pixel space with RED values.
  for(int x=0;x<gl_columns;x++){
    for(int y=0;y<gl_rows;y++){
      int plc = y*gl_columns*4+x*4;
      f[plc  ] = 1;
      f[plc+1] = 0;
      f[plc+2] = 0;
      f[plc+2] = 1;
    }
  }

  
  // then fill a subset of the buffer with the magick pixel values.
  for(int x=0;x<columns();x++){
    for(int y=0;y<rows();y++){
      ColorRGB c(pixels[y*columns()+x]);
      int plc = y*gl_columns*4+x*4;
      if(!alphaOnly){
	f[plc  ] = c.red();
	f[plc+1] = c.green();
	f[plc+2] = c.blue();
      }else{
	f[plc  ] = 1;
	f[plc+1] = 1;
	f[plc+2] = 1;
      }
      f[plc+3] = 1-c.alpha();
    }
  }
  
    
  //CREATE TEXTURE
  glGenTextures(1,&textureID);
  glBindTexture(GL_TEXTURE_2D, textureID);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
  gluBuild2DMipmaps( GL_TEXTURE_2D, 4, gl_columns,gl_rows,GL_RGBA, GL_FLOAT, f );
  glTexImage2D(GL_TEXTURE_2D, 0,4, gl_columns,gl_rows, 0, GL_RGBA, GL_FLOAT, f);
  
  //CLEAN
  delete [] f;
  
  //USE TEXTURE TO MAKE A DISPLAY LIST
  listID = glGenLists(1);
  glNewList(listID, GL_COMPILE);
  glBindTexture(GL_TEXTURE_2D, textureID);
  

  //ratio may not be square.
  
  float w;
  float h;
  
  /*
  if(gl_rows<gl_columns){
    w = 1;
    h = (float)rows()/(float)columns();
  }else{
    w = (float)columns()/(float)rows();
    h = 1;
  }
  */
  w=1;
  h=1;
  
  glBegin(GL_QUADS);
  glTexCoord2f(0,0);glVertex2f(0,0);	
  glTexCoord2f(1,0);glVertex2f(w,0);
  glTexCoord2f(1,1);glVertex2f(w,h);
  glTexCoord2f(0,1);glVertex2f(0,h);
  glEnd();
  
  glBindTexture(GL_TEXTURE_2D,0);
  glEndList();
  glBindTexture(GL_TEXTURE_2D,0);
}


JImage::~JImage(){
  deleteGLResources();
}

/**
   wrapper for glCallList
*/
void JImage::drawList(){
  glCallList( listID );
}


void JImage::draw(){
  glPushMatrix();
  int w = getNextHighestGLTexSize();
  glScalef(w,w,w);
  drawList();
  glPopMatrix();
}



void JImage::deleteGLResources(){
  if(listID!=0){
    glDeleteLists(listID,1);
    glDeleteTextures(1,&textureID);
    listID=0;
    textureID=0;
  }
}
