#ifndef _JImage_H_
#define _JImage_H_

#include <Magick++.h>
#include "jttoolkit.h"

using namespace Magick;
using namespace std;


/**
   PImage-esque support for photos in space. Make sure your images are
   512x512 or 256x256 or 128x128, etc. according to OpenGL texture
   policy. Since JImage subclasses the Magick++ Image object, you can
   do all kinds of things to this texture. Then to update the
   display-list used by JImage::draw() , simply call JImage::update().
   @author jtnimoy
   @code
   //how to make your own JImage
   JImage img("media/myCoolPicture.png");
   img.draw();
   @endcode
*/
class JImage : public Image{
 private:
  /**
     get rid of the display list and texture associated with this
     Magick++ Image instance.
  */
  void deleteGLResources();
 public:
  /**
     pixels is an array of pixel colors.
  */
  
  PixelPacket *pixels;

    /**
     holds the GLuint returned by OpenGL whenever it regenerates the texture.
  */

  GLuint textureID;

  /**
     holds the display list GLuint returned by OpenGL
  */
  
  GLuint listID;

  /**
     call this after manipulating the pixels in order to sync up the
     opengl display version used by JImage::draw()
  */
  
  void update();
  
  /**
     function is shared by all constructors. 
  */
  void initMem();


  /**
     load a new image from just an array of pixels.
  */
  void loadMemory(unsigned char *,int w,int h);

  
  void load(char *); ///< load a new image file given a file name.

  
  /**
     load in an image that is compatible with Magick++ file input. For
     now, please make sure the image is square and that the dimensions
     are a mult of 2.
  */

  void update(int alphaOnlyBool);

  /**
     DEPRECATED - USE draw() - draw the image into the openGL space. note, you will need to glscale to the desired size before calling this func since drawList will lay out the image as a rect(0,0,1,1).
  */
  void drawList();//deprecated.

  
  /**
     renders the image to the opengl buffer, at the size at which it was imported.
  */
  void draw();

  
  /**
     calculates a suitable openGL texture.
  */
  int getNextHighestGLTexSize();

  
  /**
     empty constructor - make sure you call load soon in this case.
  */
  JImage();

  /**
     construct using an input file string. see JImage::load()
  */
  JImage(char *filename);

  
  ~JImage();
  
};
#endif
