#ifndef _JImageSequence_H_
#define _JImageSequence_H_


#include "JImage.h"
#include <vector>

using namespace std;

/**
   Draws an image sequence in some folder to stage, providing pixel
   color access. Great for footage backplate and making footage pixel
   filters.
   @author jtnimoy
*/
class JImageSequence {
 public:

  /**
     how many times has the sequence wrapped around so far?
  */
  int loopCount;

  /**
     what frame number have we stepped to?
  */
  int frameNum;

  /**
     internal counter.
  */
  int counter;

  /**
     internal frame count
  */
  int max;

  /**
     frame number offset
  */
  int begin;

  /**
     a sprintf formatted string specifying the file image sequence. 
  */
  char *filestr;

  /**
     a list of pointers to the actual JImages. These are only loaded as needed when calling draw.
   */
  vector<JImage*> frames;

  /**
     construct and load
     @param filestr sprintf formatted filename string for the sequence
     @param begin frame number offset
     @param end frame number offset (note, this is not a duration, it is begin+duration)
  */
  JImageSequence(char *filestr, int begin,int end);
  
  /**
     empty constructor for static allocation - be sure to call JImageSequence::load() asap.
  */
  JImageSequence();
  ~JImageSequence();
  
  /**
     step the footage forward by one frame. this auto-wraps. also see loopCount
  */
  void step();

  /**
     step the footage backward by one frame.
  */
  void stepBack();

  /**
     render the footage as a JImage. See JImage::drawList() for more information on this rendering format.
  */
  void draw();
  
  /**
     load up a new image sequence.
     @param filestr sprintf formatted filename string for the sequence
     @param begin frame number offset
     @param end frame number offset (note, this is not a duration, it is begin+duration)
  */
  void load(char *filestr, int begin,int end);
};



#endif

