/**
   JPath 
   by Josh Nimoy
   for Gabe Dunne
   23 jan 2007
   embelleshed by evan raskob, 20 nov 2007
*/

#include "JPath.h"
#include "jttoolkit.h"
#include "Global.h"



JPath::JPath(JPath *mentor){
  JPath(mentor->points);
}


JPath::JPath(){
  resetVars();
}

JPath::JPath(deque<JPoint*>p){
  resetVars();
  
  //make a deep copy of that vector
  deque<JPoint*>::iterator it;
  for(it=p.begin();it!=p.end();it++){
    points.push_back(new JPoint(**it));
  }
}

JPath::JPath(vector<JPoint*>p){
  resetVars();
  
  //make a deep copy of that vector
  vector<JPoint*>::iterator it;
  for(it=p.begin();it!=p.end();it++){
    points.push_back(new JPoint(**it));
  }
}



void JPath::resetVars(){
  stroke_width = 1.5;
  useInnerFill = true;
  useOutline = false;
  stroke_flare = 10;
  stroke_style = 0;
}


void JPath::fromBezier(JPoint p1,JPoint p2,JPoint p3,JPoint p4,int steps){
  
  // allocate some working space.
  float *Xs = new float[steps];
  float *Ys = new float[steps];
  float *Zs = new float[steps];
  
  interpolateBezier(
		    p1.x,p1.y,p1.z,
		    p2.x,p2.y,p2.z,
		    p3.x,p3.y,p3.z,
		    p4.x,p4.y,p4.z,
		    steps,Xs,Ys,Zs) ;
  
  
  points.push_back(new JPoint(p1));
  for(int i=1;i<steps-1;i++){
    points.push_back(new JPoint(Xs[i],Ys[i],Zs[i]));
  }
  points.push_back(new JPoint(p4));

  delete Xs;
  delete Ys;
  delete Zs;
  
}




JPath::~JPath(){
  while(points.size()>0){
    delete points.back();
    points.pop_back();
  }
/*
  deque<JPoint*>::iterator it;
  for(it=points.begin();it!=points.end();it++){
    delete *it;
  }
  points.clear();
*/
}

/**
   divide all edges in 2.
*/
void JPath::subdiv(){
  deque<JPoint*>dest;
  
  deque<JPoint*>::iterator it;
  int i=0;
  for(it=points.begin();it!=points.end();it++){
    dest.push_back(*it);
    if(i<points.size()-1){//don't add one for the last
      dest.push_back(new JPoint( (*it)->lerpTo(**(it+1),0.5)    ));
    }
    i++;
  }
  
  //ok, now copy dest back into points
  points.clear();
  for(it=dest.begin();it!=dest.end();it++){
    points.push_back(*it);
  }
  
}

/**
  Add a new JPoint to this path in the back
  Is this a correct use of pointers?
*/
void JPath::pushPointBack (JPoint p)
{
  if (points.size() > MAX_PATH_POINTS)
  {
	 delete (JPoint*)points.front();
	 points.pop_front();
  }
  points.push_back(new JPoint(p));
}


/**
  Add a new JPoint to this path in the front
  Is this a correct use of pointers?
**/
void JPath::pushPointFront(JPoint p) 
{

  if (points.size() > MAX_PATH_POINTS)
  {
	 delete (JPoint*)points.back();
	 points.pop_back();
  }
  points.push_front(new JPoint(p));
}




/**
   loop through all the verts and call glVertex3f(x,y,z);
**/
void JPath::glVertex(){
  deque<JPoint*>::iterator it;
  for(it=points.begin();it!=points.end();it++){
    (*it)->glVertex();
  }
  
}


/*
  lerping towards the previous and next points, we are effectively
  melting the geometry toward the center. 
*/
void JPath::melt(float heat){
  //draw handles

  deque<JPoint*>::iterator it;
  int i=0;
  for(it=points.begin();it!=points.end();it++){
    
    
    JPoint *curr = *it;
    JPoint *prev;
    JPoint *next;
    
    if(i>0)prev=*(it-1);
    else   prev=curr;
    
    if(i<points.size()-1)next=*(it+1);
    else   next=curr;
    
    curr->lerpSelfTo(*prev,heat);
    curr->lerpSelfTo(*next,heat);
    
    i++; //keep this close to the end curly bracket.
  }
}

void JPath::smooth(int val){
  for(int i=0;i<val;i++){
    subdiv();
    melt(0.5);
  }
}

void JPath::println(){
  
  cout << "JPath( ";
  deque<JPoint*>::iterator it;
  for(it=points.begin();it!=points.end();it++){
    JPoint *t = *it;
    cout << "JPoint(" << t->x << ' ' << t->y << ' ' << t->z << ") ";
  }
  cout << " ) " << endl;
}

void JPath::render(){
  deque<JPoint*>::iterator it;
  glBegin(GL_LINE_STRIP);
  for(it=points.begin();it!=points.end();it++){
    (**it).glVertex();
  }
  glEnd();
}

void JPath::reset(){
  //kill the entire memory.
  while(points.size()>0){
    delete points.back();
    points.pop_back();
  }
}

void JPath::renderAsBezier(int steps){
  JPath p;
  p.fromBezier(*points[0],*points[1],*points[2],*points[3],steps);
  p.render();
}


void JPath::stroke1(int steps, bool bez=false)
{
  if (points.size() < 2) return;
  deque<JPoint*>::iterator it;
  JPath *p;
  if(bez)
  {
    p = new JPath();
    p->fromBezier(*points[0],*points[1],*points[2],*points[3],steps);
  }
  else p = new JPath(points);
  
  deque<JPoint>row1;
  deque<JPoint>row2;
  deque<JPoint>row3;
  
  int i=0;
  for(it=p->points.begin();it!=p->points.end();it++)
  {
    JPoint p1;
    JPoint p2;
    JPoint p3;
    JPoint p4;
  
    if(i==0){//beginning case

      p1.copyFrom(**(it));
      p2.copyFrom(**(it+1));
      p3.copyFrom(p2);
      p4.copyFrom(p2);
      p3.rotateSelfZ( PI/2,p1);
      p4.rotateSelfZ(-PI/2,p1);
	  
	} else if(i==points.size()-1){//end case
	
      p1.copyFrom(**(it-1));
      p2.copyFrom(**(it));
      p3.copyFrom(p1);
      p4.copyFrom(p1);
      p3.rotateSelfZ(-PI/2,p2);
      p4.rotateSelfZ( PI/2,p2);
	  
    } else {//in the middle, we average the angles.

      p1.copyFrom(**(it-1));
      p2.copyFrom(**(it));
      p3.copyFrom(p1);
      p4.copyFrom(p1);
      p3.rotateSelfZ(-PI/2,p2);
      p4.rotateSelfZ( PI/2,p2);
      
      //calculate averagables
      JPoint p5(p1);
      JPoint p6(p1);
      p5.rotateSelfZ(-PI/2,p2);
      p6.rotateSelfZ( PI/2,p2);
      p3.lerpSelfTo(p5,0.5);
      p4.lerpSelfTo(p6,0.5);
    }
	
	
    //accumilate the results for later.
    row1.push_back(JPoint(p3));
    row2.push_front(JPoint(p4));
    row3.push_back(JPoint(**it));

    ////normalise the outstretches away from the skel.
    //row1.back().normalizeSelfFrom(row3.back(),10);
    //row2.front().normalizeSelfFrom(row3.back(),10);
    i++;
  }
	
  delete p;
  
  //MASSAGE THE SHAPE
  for(int i=0;i<row1.size();i++){
	
    int ii = row1.size()-1-i;


    //  >>===---===<<
    //float fatness1 = stroke_width*2-pow((stroke_width*(float)i)/row1.size() * (stroke_width*(float)ii)/row1.size(),2);
    float fatness1 = stroke_width*stroke_flare -
      (stroke_width * ((float)i/row1.size() * (stroke_width*(float)ii) / row1.size()));
    
    
    
    //   ---==+++==--
    float fatness2 = (stroke_width*(float)i)/row1.size() * (stroke_width*(float)ii)/row1.size();
    //float fatness3 = row1[i].distanceFrom( row2[row2.size()-1-i-1]);
    float fatness3 = row1[i].distanceFrom( row2[row2.size()-1-i]);
    
    float fatness;
    if(stroke_style==0){
      fatness = fatness1*points.size();
    }else if(stroke_style==1){      
      fatness = fatness3*points.size()*0.001;
    }else{   
      float p =points.size()*0.1;
      fatness =  fmax(1,(stroke_width*100 - fmin(stroke_width*100 ,fatness3*2)))*0.5;
    }
    
    row1[ i].normalizeSelfFrom(row3[i],fatness);
    row2[ii].normalizeSelfFrom(row3[i],fatness);
    
    //magnets to buckle down the edges
  }  
  
  //draw buff
  
  
  if (row1.size() > 2)
  {
	int passes = 0;

	if ( ((float)row1.size() / 3.0) == 0.0)
		passes = row1.size();
	else 
		passes = row1.size()  - (row1.size() % 3);


	  if(useInnerFill){
		glBegin(GL_TRIANGLE_STRIP);
		for(int i=0; i < passes; i++){
		  row2[row2.size()-1-i].glVertex();
		  row1[i].glVertex();
		}
		glEnd();
	  }
  }
  
  if(useOutline){
    glBegin(GL_LINE_LOOP);
    for(int i=0;i<row1.size();i++)row1[i].glVertex();
    for(int i=0;i<row2.size();i++)row2[i].glVertex();
    glEnd();
  }
  
  
  /*
  if (row1.size() > 1)
  {
	int passes = 2;

	if ( ((float)row1.size() / 2.0) == 0.0)
		passes = row1.size();
	else 
		passes = row1.size() - 1;
  
	  
	  //draw ladder spokes.
	  glBegin(GL_LINES);
	  for(int i=0; i < passes; i++){
		cout << i << "/" << passes << endl;
		
		row1[i].glVertex();
		row2[passes-i-1].glVertex();
	  }
	  glEnd();
	}
  */
  
  //cout<< "got this far 10" << endl;
  
  //draw middle skeleton
  /*
  glBegin(GL_LINE_STRIP);
  for(int i=0;i<row3.size();i++){
    row3[i].glVertex();
  }
  glEnd();
  */
  
}



void JPath::stroke(float line_width=1.0)
{
  
  if (points.size() < 2) return;
  
  deque<JPoint*>::iterator it;
  
  deque<JPoint>row1;
  deque<JPoint>row2;

  JPoint p1;
  JPoint p2;
  JPoint p3;
  JPoint p4;
  
  int i=0;
  
  
  for(it=points.begin(); it!=points.end(); it++)
  {
	p1.copyFrom(**(it));
    
    if ((i % 2) == 0)
	{
	  p1.rotateSelfZ( PI/2,p1);
	}
    row1.push_back(p1);
    i++;  
  }
  
  /*
  for(it=points.begin(); it!=points.end(); it++)
  {

    if(i==0) {//beginning case

      p1.copyFrom(**(it));
      p2.copyFrom(**(it+1));
      p3.copyFrom(p2);
      p4.copyFrom(p2);
	  
      p3.rotateSelfZ( PI/2,p1);
      p4.rotateSelfZ(-PI/2,p1);
	  
	} else if(i==points.size()-1){//end case
	
      p1.copyFrom(**(it-1));
      p2.copyFrom(**(it));
      p3.copyFrom(p1);
      p4.copyFrom(p1);
      p3.rotateSelfZ(-PI/2,p2);
      p4.rotateSelfZ( PI/2,p2);
	  
    } else {//in the middle, we average the angles.

      p1.copyFrom(**(it-1));
      p2.copyFrom(**(it));
      p3.copyFrom(p1);
      p4.copyFrom(p1);
      p3.rotateSelfZ(-PI/2,p2);
      p4.rotateSelfZ( PI/2,p2);
      
      //calculate averagables
      JPoint p5(p1);
      JPoint p6(p1);
      p5.rotateSelfZ(-PI/2,p2);
      p6.rotateSelfZ( PI/2,p2);
      p3.lerpSelfTo(p5,0.5);
      p4.lerpSelfTo(p6,0.5);
    }
	
	
    //accumilate the results for later.
    row1.push_back(JPoint(p3));
    row2.push_front(JPoint(p4));
    row3.push_back(JPoint(**it));

    ////normalise the outstretches away from the skel.
    //row1.back().normalizeSelfFrom(row3.back(),10);
    //row2.front().normalizeSelfFrom(row3.back(),10);
    i++;
  }
 
	  
  //MASSAGE THE SHAPE
  for(int i=0;i<row1.size();i++) {
	
	  int ii = row1.size()-1-i;

	  //  >>===---===<<
	  //float fatness1 = stroke_width*2-pow((stroke_width*(float)i)/row1.size() * (stroke_width*(float)ii)/row1.size(),2);
	  float fatness1 = stroke_width*stroke_flare -
		  (stroke_width * ((float)i/row1.size() * (stroke_width*(float)ii) / row1.size()));
		
		
		
	  //   ---==+++==--
	  float fatness2 = (stroke_width*(float)i)/row1.size() * (stroke_width*(float)ii)/row1.size();
	  //float fatness3 = row1[i].distanceFrom( row2[row2.size()-1-i-1]);
	  float fatness3 = row1[i].distanceFrom( row2[row2.size()-1-i]);
		
	  float fatness;
	  if(stroke_style==0){
		  fatness = fatness1*points.size();
	  }else if(stroke_style==1){      
		  fatness = fatness3*points.size()*0.001;
	  }else{   
		  float p =points.size()*0.1;
		  fatness =  fmax(1,(stroke_width*100 - fmin(stroke_width*100 ,fatness3*2)))*0.5;
	  }
		
	  row1[ i].normalizeSelfFrom(row3[i],fatness);
	  row2[ii].normalizeSelfFrom(row3[i],fatness);
    
      //magnets to buckle down the edges
  }  
  
  //draw buff
  */


  if (row1.size() > 2)
  {
	int passes = 0;

	if ( ((float)row1.size() / 2.0) == 0.0)
		passes = row1.size();
	else 
		passes = row1.size()  - (row1.size() % 2);


	  if(useInnerFill){
		glBegin(GL_LINES);
		for(int i=0; i < passes; i++){
		  //row2[row2.size()-1-i].glVertex();
		  row1[i].glVertex();
		}
		glEnd();
	  }
  }

}



void JPath::drink(FingerPoints f, int offset){
  deque<JPoint*>::iterator it;
  int i=0;
  for(it=points.begin();it!=points.end();it++){
    (**it)+=f[offset+i];
    i++;
  }
}



JPoint JPath::average(){
  JPoint accum;
  deque<JPoint*>::iterator it;
  for(it=points.begin();it!=points.end();it++){
    accum+=(**it);
  }
  return accum/(float)points.size();
}


void JPath::shrink(float v){
  JPoint accum;
  JPoint a = average();
  deque<JPoint*>::iterator it;
  for(it=points.begin();it!=points.end();it++){
    (**it).lerpSelfTo(a,v);
  }

}

void JPath::operator+=(JPoint p){
  deque<JPoint*>::iterator it;
  for(it=points.begin();it!=points.end();it++){
    (**it)+=p;
  }
}

void JPath::operator*=(JPoint p){
  deque<JPoint*>::iterator it;
  for(it=points.begin();it!=points.end();it++){
    (**it)*=p;
  }
}


void JPath::eatPointsFromBothEnds(int bites){
  for(int i=0;i<bites;i++){
    
    if(points.size()>0){
      delete (JPoint*)points.front();
      points.pop_front();
    }
    
    if(points.size()>0){
      delete (JPoint*)points.back();
      points.pop_back();
    }
    
  }
}


void JPath::extrudePsychedelic(){
  lights();
  glBegin(GL_TRIANGLE_STRIP);
  float rad = stroke_width*0.4*points.size();
  float s = 316*0.01;
  deque<JPoint*>::iterator it;
  int i=0;
  for(it=points.begin();it!=points.end();it++){
    JPoint p(**it);
    p+=JPoint(cos(i*s)*rad,0,sin(i*s)*rad);
    if(i>0){
      JPoint pp(p-**(it-1));
      glNormal3f(pp.x,pp.y,pp.z);
    }
    
    p.glVertex();
    i++;
  }
  
  glEnd();
  nolights();
}
